@@ -4,6 +4,7 @@ To use it you must have IPFS running on your computer.
 Configure IPFS to enable all this:
 ipfs config --json Experimental.Libp2pStreamMounting true
 """
+import shutil
 from queue import Queue
 import socket
 import threading
@@ -281,9 +282,6 @@ class TransmissionListener:
         except:
             pass
 
-    def __del__(self):
-        self.Terminate()
-
 
 def StartConversation(conversation_name,
                       peerID,
@@ -696,15 +694,12 @@ class Conversation:
             transmission_send_timeout_sec=transmission_send_timeout_sec,
             transmission_request_max_retries=transmission_request_max_retries)
 
-    def Terminate(self):
+    def Close(self):
         if self.listener:
             self.listener.Terminate()
 
-    def Close(self):
-        self.Terminate()
-
     def __del__(self):
-        self.Terminate()
+        self.Close()
 
 
 class ConversationListener:
@@ -759,9 +754,6 @@ class ConversationListener:
     def Terminate(self):
         self.listener.Terminate()
 
-    def __del__(self):
-        self.Terminate()
-
 
 def TransmitFile(filepath,
                  peerID,
@@ -985,10 +977,6 @@ class FileTransmitter:
         if info[0].decode('utf-8') == "ready":
             self.StartTransmission()
 
-    def __del__(self):
-        if self.conversation:
-            self.conversation.Close()
-
 
 class FileTransmissionReceiver:
     transmission_started = False
@@ -1029,7 +1017,7 @@ class FileTransmissionReceiver:
                 self.filesize = FromB255No0s(filesize)
                 self.filename = filename.decode('utf-8')
                 self.metadata = metadata
-                self.writer = open(os.path.join(self.dir, self.filename), "wb")
+                self.writer = open(os.path.join(self.dir, self.filename + ".PART"), "wb")
                 self.transmission_started = True
                 self.conv.Say("ready".encode())
                 self.writtenbytes = 0
@@ -1068,6 +1056,8 @@ class FileTransmissionReceiver:
 
     def Finish(self):
         self.writer.close()
+        shutil.move(os.path.join(self.dir, self.filename + ".PART"),
+                    os.path.join(self.dir, self.filename))
         if print_log:
             print("FileReception: " + self.filename
                   + ": Transmission finished.")
@@ -1081,9 +1071,6 @@ class FileTransmissionReceiver:
             else:
                 self.eventhandler(self.conv.peerID, filepath)
 
-    def __del__(self):
-        self.conv.Close()
-
 
 # ----------IPFS Technicalitites-------------------------------------------
 connections_send = list()
@@ -1180,20 +1167,20 @@ def CloseListeningConnection(protocol, port):
         protocol=f"/x/{protocol}", targetaddress=f"/ip4/127.0.0.1/tcp/{port}")
 
 
-def ListenToBuffers(eventhandler,
-                    proto,
-                    buffer_size=def_buffer_size,
-                    monitoring_interval=2,
-                    status_eventhandler=None,
-                    eventhandlers_on_new_threads=True):
-    return BufferReceiver(
-        eventhandler,
-        proto,
-        buffer_size=buffer_size,
-        monitoring_interval=monitoring_interval,
-        status_eventhandler=status_eventhandler,
-        eventhandlers_on_new_threads=eventhandlers_on_new_threads
-    )
+def ListenToBuffersOnPort(eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None):
+    return Listener(eventhandler, port, buffer_size, monitoring_interval, status_eventhandler)
+
+
+def ListenToBuffers(eventhandler, proto, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None, eventhandlers_on_new_threads=True):
+    listener = ListenerTCP(eventhandler, 0, buffer_size,
+                           eventhandlers_on_new_threads=eventhandlers_on_new_threads)
+    CreateListeningConnection(proto, listener.port)
+    return listener
+
+
+def SendBufferToPort(buffer, addr, port):
+    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+    sock.sendto(buffer, (addr, port))
 
 
 class BufferSender():
@@ -1210,44 +1197,6 @@ class BufferSender():
             self.sock = CreateSendingConnection(self.peerID, self.proto)
             self.sock.send(data)
 
-    def Terminate(self):
-        CloseSendingConnection(self.peerID, self.proto)
-
-    def __del__(self):
-        self.Terminate()
-
-
-class BufferReceiver():
-    def __init__(self,
-                 eventhandler,
-                 proto,
-                 buffer_size=def_buffer_size,
-                 monitoring_interval=2,
-                 status_eventhandler=None,
-                 eventhandlers_on_new_threads=True):
-        self.proto = proto
-        self.listener = ListenerTCP(
-            eventhandler,
-            0,
-            buffer_size=buffer_size,
-            monitoring_interval=monitoring_interval,
-            status_eventhandler=status_eventhandler,
-            eventhandlers_on_new_threads=eventhandlers_on_new_threads
-        )
-        CreateListeningConnection(proto, self.listener.port)
-
-    def Terminate(self):
-        CloseListeningConnection(self.proto, self.listener.port)
-        self.listener.Terminate()
-
-    def __del__(self):
-        self.Terminate()
-
-
-def SendBufferToPort(buffer, addr, port):
-    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-    sock.sendto(buffer, (addr, port))
-
 
 class ListenerTCP(threading.Thread):
     """
@@ -1259,8 +1208,7 @@ class ListenerTCP(threading.Thread):
             print("Received data from  " + sender_peerID)
             print(data.decode("utf-8"))
 
-        # start listening to incoming buffers on an automatically assigned port (that's what the 0 means)
-        listener = Listener(eventhandler, 0, 2048)
+        listener = Listener(eventhandler, 0, 2048)   # start listening to incoming buffers on an automatically assigned port (that's what the 0 means)
         port = listener.port    # retrieve the automatically assigned port
 
         # Once finished and listening on that port should be stopped:
@@ -1278,20 +1226,14 @@ class ListenerTCP(threading.Thread):
     sock = None
     last_time_recv = datetime.utcnow()
 
-    def __init__(self,
-                 eventhandler,
-                 port=0,
-                 buffer_size=def_buffer_size,
-                 monitoring_interval=2,
-                 status_eventhandler=None,
-                 eventhandlers_on_new_threads=True):
+    def __init__(self, eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None, eventhandlers_on_new_threads=True):
         threading.Thread.__init__(self)
         self.port = port
         self.eventhandler = eventhandler
         self.buffer_size = buffer_size
         self.eventhandlers_on_new_threads = eventhandlers_on_new_threads
         self.sock = socket.socket(
-            socket.AF_INET, socket.SOCK_STREAM)
+            socket.AF_INET, socket.SOCK_STREAM)      # For UDP
 
         self.sock.bind(("127.0.0.1", self.port))
         # in case it had been 0 (requesting automatic port assiggnent)
@@ -1332,6 +1274,7 @@ class ListenerTCP(threading.Thread):
                     self.eventhandler(data)
         conn.close()
         self.sock.close()
+        CloseListeningConnection(str(self.port), self.port)
         if print_log_connections:
             print("Closed listener.")
 
@@ -1354,11 +1297,8 @@ class ListenerTCP(threading.Thread):
         if print_log_connections:
             print("terminating listener")
         self.terminate = True   # marking the terminate flag as true
-        # SendBufferToPort("end".encode(), "127.0.0.1", self.port)
         self.sock.close()
-
-    def __del__(self):
-        self.Terminate()
+        # SendBufferToPort(bytearray([0]),"",self.port) # to make the listener's buffer receiving while loop move forwards so that it realises it has o stop
 
 
 class Listener2TCP(threading.Thread):
@@ -1631,10 +1571,6 @@ class Listener2(threading.Thread):
         # SendBufferToPort(bytearray([0]),"127.0.0.1",self.port) # to make the listener's buffer receiving while loop move forwards so that it realises it has o stop
 
 
-def ListenToBuffersOnPort(eventhandler, port=0, buffer_size=def_buffer_size, monitoring_interval=2, status_eventhandler=None):
-    return Listener(eventhandler, port, buffer_size, monitoring_interval, status_eventhandler)
-
-
 def AddIntegrityByteToBuffer(buffer):
     # Adding an integrity byte that equals the sum of all the bytes in the buffer modulus 256
     # to be able to detect data corruption:
